// src/backend/dynamic_schedulable.hh - Copyright 2005, 2006, University
//               of Padova, dept. of Pure and Applied
//               Mathematics
//
// This file is part of SGPEMv2.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// SGPEMv2 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with SGPEMv2; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#ifndef DYNAMIC_SCHEDULABLE_HH
#define DYNAMIC_SCHEDULABLE_HH 1

#include "config.h"
#include "schedulable.hh"
#include "static_schedulable.hh"
#include "../templates/smartp.hh"

namespace sgpem
{
  class DynamicSchedulable;

  /** \brief Desribes the state of a schedulable entity in a particular moment
   * of the simulation
   *
   * This class stores part of the information deeded by the Scheduler to
   * manage processes and other ones.
   *
   * Objects of type DynamicSchedulable are created by the Scheduler and are
   * destroyed by SimulationStatus if they are linked to it or by the Scheduler.
   */
  class SG_DLLEXPORT DynamicSchedulable : public virtual Schedulable
  {
  public:
    /** \brief Object constructor */
    DynamicSchedulable(StaticSchedulable& obj);

    //DynamicSchedulable(const DynamicSchedulable& obj);	//copy constructor

    /** \brief Verify if two instances represents the same situation
     *
     * This trivial overloading of the equal operator simply verifies if the
     * actual represented process is the same, and if the status is also the
     * same.
     */
    bool operator==(const DynamicSchedulable&) const;

    Glib::ustring get_name() const;

    void set_name(const Glib::ustring& new_name);

    unsigned int get_arrival_time() const;

    int get_base_priority() const;

    unsigned int get_total_cpu_time() const;

    int get_priority_push() const;

    void set_priority_push(int new_value = 0);

    int get_current_priority() const;

    unsigned int get_remaining_time() const;

    void decrease_remaining_time();

    int get_last_acquisition() const;

    void set_last_acquisition(int instant);

    int get_last_release() const;

    void set_last_release(int instant);


    /* 
                  FIXME
      all following methods are deprecated, drop them
    */
  
    /** \brief Returns the remaining CPU time */
    int get_cpu_time_left() const;

    /** \brief Decrements cpu time left by this amount until it reaches zero */
    void give_cpu_time(const int& time);

    /** \brief Sets the time when this process was last scheduled */
    void set_last_scheduled(const int& time);

    /** \brief Gets the time when this process was last scheduled */
    int get_last_scheduled() const;

    /** \brief Returns the schedule stack in which this process resides
     * \see state */
    state get_state() const;

    /** \brief Sets the state of this process
     * \see state 
     */
    void set_state(state s);

    void serialize(SerializeVisitor& translator) const
    {
    }
    
    /** \brief Returns a pointer to the schedulable object
     *
     * This function returns a pointer to the actual schedable object
     * represented, along with its status, by this instance.
     */
    StaticSchedulable* get_schedulable() const;

  protected:
    int _time_left;
    memory::smart_ptr<StaticSchedulable> _ref;

  private:
    int _last_acquisition;
    int _last_release;
    int _priority_push;

    //FIXME deprecated stuff used by deprecated methods    
    int _last;
    state _my_state;
  };
}

#endif
